/*
	Copyright (C) 2008 David Fleer.

	This file is part of mkesa.

	mkesa is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 2 of the License, or
	(at your option) any later version.

	mkesa is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with mkesa.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _MKESA_QSORTS_H_
#define _MKESA_QSORTS_H_

#include <libfid.h>

#ifdef MKESA_DATA64
#include <libfid.64>
#else
#include <libfid.32>
#endif

#define min(a, b)	(a) < (b) ? a : b

/*
 * Qsort from Bentley & McIlroy's "Engineering a Sort Function".
 * Swaps an entire section of an array.
 */
#define swapcode(TYPE, parmi, parmj, n) { 		\
	long i = n; 			\
	TYPE *pi = (TYPE *) (parmi); 		\
	TYPE *pj = (TYPE *) (parmj); 		\
	do { 						\
		TYPE temp = *pi;		\
		*pi++ = *pj;				\
		*pj++ = temp;				\
        } while (--i > 0);				\
}

/* Swap macro */
#define swap(a, b) 					\
		t = *(UINT *)(a);			\
		*(UINT *)(a) = *(UINT *)(b);		\
		*(UINT *)(b) = t;

#define vecswap(a, b, n) 	if ((n) > 0) swapcode(UINT, a, b, n)

#define MED_OF_3_BODY(CMPFUN,D)\
	const fid_Symbol* base = tp + (D);\
	\
	if((CMPFUN)(base + *a, base + *b) == 0) return a;\
	if((CMPFUN)(base + *a, base + *c) == 0 ||\
	   (CMPFUN)(base + *b, base + *c) == 0)\
		return c;\
	\
	return (\
			 (CMPFUN)(base + *a, base + *b) < 0 ?\
			((CMPFUN)(base + *b, base + *c) < 0 ? b : \
			((CMPFUN)(base + *a, base + *c) < 0 ? c : a )) :\
			((CMPFUN)(base + *b, base + *c) > 0 ? b : \
			((CMPFUN)(base + *a, base + *c) < 0 ? a : c )))

/* Median of three suffixes using normal comparison */
static inline const UINT* med3(const UINT *restrict a, 
		const UINT *restrict b, const UINT *restrict c,
		const fid_Symbol *restrict tp, const UINT lcp)
{
	MED_OF_3_BODY(cmp_suf_lcp_ro,lcp);
}

/* Median of three, comparing only the depth's symbol of 
 * each suffix */
static inline const UINT* med3_depth(const UINT *restrict a,
		const UINT *restrict b, const UINT *restrict c,
		const fid_Symbol *restrict tp, const UINT depth)
{
	MED_OF_3_BODY(cmp_suf_depth,depth);
}

/* Median of three based on the offset alone */
static inline const UINT* med3_offset(const UINT *a, const UINT *b,
		const UINT *c)
{
	return ((*a < *b) ? 
			((*b < *c) ? b : 
			((*a < *c) ? c : a )) :
			((*b > *c) ? b : 
			((*a < *c) ? a : c )));
}

static inline const UINT* med3_rank(const UINT *a, const UINT *b,
		const UINT *c, const UINT *sufarray)
{
	return ((sufarray[*a] < sufarray[*b]) ? 
			((sufarray[*b] < sufarray[*c]) ? b : 
			((sufarray[*a] < sufarray[*c]) ? c : a )) :
			((sufarray[*b] > sufarray[*c]) ? b : 
			((sufarray[*a] < sufarray[*c]) ? a : c )));
}

void qsort_suf(UINT *a, size_t n, 
		const sort_params_t* sp, UINT lcp);

void multikey_qsort_suf(UINT *a, size_t n, 
		const sort_params_t* sp, UINT depth);

void qsort_offset(UINT *a, size_t n);

void qsort_rank(UINT *a, size_t n, const UINT* sufarray);

#endif
