/*
	Copyright (C) 2008, 2009 David Fleer.

	This file is part of mkesa.

	mkesa is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 2 of the License, or
	(at your option) any later version.

	mkesa is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with mkesa.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _MKESA_THREADS_H_
#define _MKESA_THREADS_H_

#include <libfid.h>

#ifdef MKESA_DATA64
#include <libfid.64>
#else
#include <libfid.32>
#endif

#ifdef MULTITHREADED
#include <pthread.h>

#ifdef HAVE_SEMAPHORE_H
#include <semaphore.h>
#endif

#endif

typedef struct {
	fid_Symbol bucket[2];
	UINT lcp;
	size_t n;
} sort_job_t;

typedef struct {
  sort_params_t* sp;

	sort_job_t* job_queue;
	pthread_mutex_t* queue_lock;
	sem_t* queue_semaphore;
	fid_Uint16* queue_pos;
	fid_Uint16 queue_size;
	
	fid_Uint16* ready_count;
	pthread_mutex_t* ready_lock;
	pthread_cond_t* ready_cond;

} thread_arg_t;

void* start_sort_thread(void* options);

pthread_t* start_threads(thread_arg_t* thread_args,
    const fid_Uint16 thread_count, sort_params_t* sp);

void kill_threads(pthread_t* threads,
    thread_arg_t* thread_args, const fid_Uint16 thread_count);

#ifdef HAVE_SYSCONF
unsigned int get_num_of_cpus(void);
#endif

#endif
