/*
	Copyright (C) 2008, 2009 David Fleer.

	This file is part of mkesa.

	mkesa is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 2 of the License, or
	(at your option) any later version.

	mkesa is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with mkesa.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "util.h"
#include "common.h"

inline void* malloc_safe(size_t size) {
	void* mem = malloc(size);

	if(mem == NULL) {

		#ifdef HAVE_ERRNO_H
			fprintf(stderr,
"\nFatal error: Could not allocate %lu bytes of memory (%s).\n",
(unsigned long) size, strerror(errno));
		exit(errno);

		#else
			fprintf(stderr,
"\nFatal error: Could not allocate %lu bytes of memory.\n",
(unsigned long) size);
		exit(-1);
		#endif
	}

	return mem;
}

inline void* realloc_safe(void* ptr, size_t size) {
	void* mem = realloc(ptr, size);

	if(mem == NULL) {

		#ifdef HAVE_ERRNO_H
			fprintf(stderr,
"\nFatal error: Could not re-allocate %lu bytes of memory (%s).\n",
(unsigned long) size, strerror(errno));
		exit(errno);

		#else
			fprintf(stderr,
"\nFatal error: Could not re-allocate %lu bytes of memory.\n",
(unsigned long) size);
		exit(-1);
		#endif
	}

	return mem;
}

int test_file_readable(const char* filebasename, const char* ext) {
	assert(filebasename != NULL);

	FILE* testfile;
	char* filename;
	int retcode;

	filename = fid_filename_create(filebasename, ext, NULL);

	if(filename == NULL) {
		fprintf(stderr, "Internal error while generating file name.\n");
		return 0;
	}

	testfile = fopen(filename, "r");

	if(testfile != NULL) {
		retcode = 1;
		fclose(testfile);
	} else {
		retcode = 0;
	}

	free(filename);

	return retcode;
}
